@@ -8,7 +8,7 @@ sub query
 {
     my $self = shift;
     $$self =~ m,^([^?\#]*)(?:\?([^\#]*))?(.*)$,s or die;
-    
+
     if (@_) {
 	my $q = shift;
 	$$self = $1;
@@ -30,17 +30,20 @@ sub query_form {
         my @query;
         while (my($key,$vals) = splice(@_, 0, 2)) {
             $key = '' unless defined $key;
-	    $key =~ s/([=&%+])/$URI::Escape::escapes{$1}/g;
+	    $key =~ s/([;\/?:@&=+,\$%])/$URI::Escape::escapes{$1}/g;
+	    $key =~ s/ /+/g;
 	    $vals = [ref($vals) ? @$vals : $vals];
             for my $val (@$vals) {
                 $val = '' unless defined $val;
-		$val =~ s/([=&%+])/$URI::Escape::escapes{$1}/g;
+		$val =~ s/([;\/?:@&=+,\$%])/$URI::Escape::escapes{$1}/g;
+                $val =~ s/ /+/g;
                 push(@query, "$key=$val");
             }
         }
         $self->query(join('&', @query));
     }
     return if !defined($old) || !length($old) || !defined(wantarray);
+    return unless $old =~ /=/; # not a form
     map { s/\+/ /g; uri_unescape($_) }
          map { /=/ ? split(/=/, $_, 2) : ($_ => '')} split(/&/, $old);
 }
@@ -52,17 +55,16 @@ sub query_keywords
     my $old = $self->query;
     if (@_) {
         # Try to set query string
-	my $k;
-        $self->query(join('+', map { $k = $_;
-				     $k =~ s/%/%25/g;
-				     $k =~ s/=/%3D/g;
-                                     $k =~ s/\+/%2B/g;
-                                     $k }
-                                     @_));
+	my @copy = @_;
+	for (@copy) { s/([;\/?:@&=+,\$%])/$URI::Escape::escapes{$1}/g; }
+	$self->query(join('+', @copy));
     }
     return if !defined($old) || !defined(wantarray);
     return if $old =~ /=/;  # not keywords, but a form
     map { uri_unescape($_) } split(/\+/, $old, -1);
 }
 
+# Some URI::URL compatibility stuff
+*equery = \&query;
+
 1;
